package com.tencent.igame.tip_flutter_image_view;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.Log;

import com.bumptech.glide.Glide;
import com.bumptech.glide.gifdecoder.GifDecoder;
import com.bumptech.glide.load.resource.gif.GifDrawable;
import com.bumptech.glide.request.FutureTarget;

import java.lang.reflect.Field;
import java.util.concurrent.ExecutionException;

/**
 * gif
 */
public class GifBitmapProvider implements BitmapProvider {
    private final Context context;
    private final String imageUrl;
    private final int width;
    private final int height;
    private final boolean centerCrop;

    private Bitmap bitmap;

    private FutureTarget<GifDrawable> gifDrawableFutureTarget;

    private GifDrawable gifDrawable;
    private GifDecoder gifDecoder;
    private Drawable.Callback callback = new Drawable.Callback() {
        @Override
        public void invalidateDrawable(Drawable who) {
        }

        @Override
        public void scheduleDrawable(Drawable who, Runnable what, long when) {
        }

        @Override
        public void unscheduleDrawable(Drawable who, Runnable what) {
        }
    };


    public GifBitmapProvider(Context context, String imageUrl, int width, int height, boolean centerCrop) {
        this.context = context;
        this.imageUrl = imageUrl;
        this.width = width;
        this.height = height;
        this.centerCrop = centerCrop;
        init();
    }

    @Override
    public void init() {
        if (centerCrop) {
            gifDrawableFutureTarget = Glide.with(context).asGif().load(this.imageUrl).submit();
        } else {
            gifDrawableFutureTarget = Glide.with(context).asGif().load(this.imageUrl).submit();
        }
        try {
            gifDrawable = gifDrawableFutureTarget.get();

            //fixme https://github.com/bumptech/glide/issues/2397
            Class gifStateClass = Class.forName("com.bumptech.glide.load.resource.gif.GifDrawable$GifState");
            Field frameLoaderField = gifStateClass.getDeclaredField("frameLoader");
            frameLoaderField.setAccessible(true);
            Object frameLoader = frameLoaderField.get(gifDrawable.getConstantState());

            Class frameLoaderClass = Class.forName("com.bumptech.glide.load.resource.gif.GifFrameLoader");
            Field gifDecoderField = frameLoaderClass.getDeclaredField("gifDecoder");
            gifDecoderField.setAccessible(true);
            gifDecoder = (GifDecoder) gifDecoderField.get(frameLoader);
            // 输出私有变量的值
            //必须设置这个callback才播放gif
            gifDrawable.setCallback(callback);
            gifDrawable.setVisible(true, true);
            gifDrawable.startFromFirstFrame();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    @Override
    public Bitmap get() {
        if (bitmap == null) {
            return gifDrawable.getFirstFrame();
        } else {
            return bitmap;
        }
    }

    @Override
    public void next() {
        if (gifDecoder != null) {
            bitmap = gifDecoder.getNextFrame();
        }
    }

    @Override
    public void clean() {
        if (!gifDrawableFutureTarget.isDone()) {
            Glide.with(context).clear(gifDrawableFutureTarget);
        }
        if (gifDrawable != null) {
            gifDrawable.stop();
        }

    }
}
