package com.tencent.igame.tip_flutter_image_view;

import android.content.Context;
import android.graphics.Bitmap;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.FutureTarget;

import java.util.concurrent.ExecutionException;

/**
 * jpeg，png
 */
public class NormalBitmapProvider implements BitmapProvider {
    private final Context context;
    private final String imageUrl;
    private final int width;
    private final int height;
    private final boolean centerCrop;

    private Bitmap bitmap;

    private FutureTarget<Bitmap> bitmapFutureTarget;

    public NormalBitmapProvider(Context context, String imageUrl, int width, int height, boolean centerCrop) {
        this.context = context;
        this.imageUrl = imageUrl;
        this.width = width;
        this.height = height;
        this.centerCrop = centerCrop;
        init();
    }

    @Override
    public void init() {
        if (centerCrop) {
            bitmapFutureTarget = Glide.with(context).asBitmap().load(this.imageUrl).centerCrop().submit(this.width, this.height);
        } else {
            bitmapFutureTarget = Glide.with(context).asBitmap().load(this.imageUrl).submit(this.width, this.height);
        }
        try {
            bitmap = bitmapFutureTarget.get();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    @Override
    public Bitmap get() {
        return bitmap;
    }

    @Override
    public void next() {

    }

    @Override
    public void clean() {
        if (!bitmapFutureTarget.isDone()) {
            Glide.with(context).clear(bitmapFutureTarget);
        }
    }
}
