package com.tencent.igame.tip_flutter_image_view;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.FutureTarget;
import com.tencent.igame.tip_flutter_image_view.webp.WebpDrawable;

import java.util.concurrent.ExecutionException;

/**
 * webp
 */
public class WebpBitmapProvider implements BitmapProvider {

    private final Context context;
    private final String imageUrl;
    private final int width;
    private final int height;
    private final boolean centerCrop;

    private Bitmap bitmap;
    private WebpDrawable webpDrawable;
    FutureTarget<WebpDrawable> bitmapFutureTarget;

    public WebpBitmapProvider(Context context, String imageUrl, int width, int height, boolean centerCrop) {
        this.context = context;
        this.imageUrl = imageUrl;
        this.width = width;
        this.height = height;
        this.centerCrop = centerCrop;
        init();
    }

    @Override
    public void init() {
        // FIXME: 2019-12-09 不能使用centerCrop
        bitmapFutureTarget = Glide.with(context).as(WebpDrawable.class).load(this.imageUrl).submit(this.width, this.height);
        try {
            webpDrawable = bitmapFutureTarget.get();
            webpDrawable.start();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    @Override
    public Bitmap get() {
        if (bitmap == null && webpDrawable != null) {
            return webpDrawable.currentBitmap;
        }
        return bitmap;
    }

    @Override
    public void next() {
        if (webpDrawable != null) {
            bitmap = webpDrawable.currentBitmap;
        }
    }

    @Override
    public void clean() {
        if (!bitmapFutureTarget.isDone()) {
            Glide.with(context).clear(bitmapFutureTarget);
        }
        if (webpDrawable != null) {
            webpDrawable.stop();
        }
    }
}
