
#import "OpenGLRender.h"
#import "NormalImageRenderWorker.h"

@interface OpenGLRender ()

@property (nonatomic, strong) NSMutableArray *workers;
@property (nonatomic, strong) NSThread *thread;

@end

@implementation OpenGLRender

static OpenGLRender *_instance = nil;

+ (OpenGLRender *)sharedRenderer {
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _instance = [[OpenGLRender alloc] init];
    });
    return _instance;
}

- (instancetype)init {
    self = [super init];

    if (self) {
        _workers = [[NSMutableArray alloc] init];
    }

    return self;
}

- (void)addWorker:(NormalImageRenderWorker *)worker {
    if (worker) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            @synchronized (self.workers) {
                [self.workers addObject:worker];
            }
        });
    }

    if (!self.thread) {
        self.thread = [[NSThread alloc] initWithTarget:self selector:@selector(run) object:nil];
        self.thread.name = @"OpenGLRender";
        [self.thread start];
    }
}

- (void)disposeWorker:(NormalImageRenderWorker *)worker {
    worker.disposed = YES;
}

- (void)run {
    while (!self.thread.cancelled) {
        CFTimeInterval loopStart = CACurrentMediaTime();

        @synchronized (self.workers) {
            BOOL hasWorkerInit = NO;
            for (NSUInteger i = 0; i < self.workers.count; ++i) {
                NormalImageRenderWorker *worker = self.workers[i];
                if (!worker.disposed) {
                    // 每帧只做一个worker的初始化，因为初始化比较慢，同时多个进行会导致滑动时卡顿
                    if (!worker.glInitDone) {
                        if (!hasWorkerInit) {
                            [worker onDraw];
                            hasWorkerInit = YES;
                        }

                    } else {
                        [worker onDraw];
                    }

                } else {
                    [worker onDispose];
                    [self removeWorker:worker];
                }
            }
        }

        CFTimeInterval waitDelta = 0.016 - (CACurrentMediaTime() - loopStart);
        if (waitDelta > 0) {
            [NSThread sleepForTimeInterval:waitDelta];
        }
    }
}

- (void)removeWorker:(NormalImageRenderWorker *)worker {
    if (worker) {
        @synchronized (self.workers) {
            [self.workers removeObject:worker];
        }
    }

    if (self.workers.count == 0) {
        [self.thread cancel];
        self.thread = nil;
    }
}

@end
