
#import "TipFlutterImageViewPlugin.h"
#import "OpenGLRender.h"
#import "NormalImageRenderWorker.h"


@interface TipFlutterImageViewPlugin ()

@property (nonatomic, strong) NSMutableDictionary<NSNumber *, NormalImageRenderWorker *> *workers;
@property (nonatomic, strong) NSObject <FlutterTextureRegistry> *textures;
@property (nonatomic, strong) dispatch_queue_t queue;

@end

@implementation TipFlutterImageViewPlugin

+ (void)registerWithRegistrar:(NSObject <FlutterPluginRegistrar> *)registrar {
    FlutterMethodChannel *channel = [FlutterMethodChannel
            methodChannelWithName:@"com.tencent.igame/flutter_image_view_method"
            binaryMessenger:[registrar messenger]];
    TipFlutterImageViewPlugin *instance = [[TipFlutterImageViewPlugin alloc] initWithTextures:[registrar textures]];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (instancetype)initWithTextures:(NSObject <FlutterTextureRegistry> *)textures {
    self = [super init];
    if (self) {
        _workers = [[NSMutableDictionary alloc] init];
        _textures = textures;
        _queue = dispatch_queue_create("method_handler_queue", DISPATCH_QUEUE_SERIAL);

    }
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    dispatch_async(self.queue, ^{
        if ([@"create" isEqualToString:call.method]) {
            NSInteger __block textureId;
            id <FlutterTextureRegistry> __weak registry = self.textures;
            NormalImageRenderWorker *worker = [[NormalImageRenderWorker alloc] initWithCall:call onNewFrame:^{
                [registry textureFrameAvailable:textureId];
            }];

            [[OpenGLRender sharedRenderer] addWorker:worker];
            textureId = (NSInteger) [self.textures registerTexture:worker];
            self.workers[@(textureId)] = worker;

            result(@(textureId));

        } else if ([@"dispose" isEqualToString:call.method]) {
            NSNumber *textureId = call.arguments[@"textureId"];
            if (![textureId isKindOfClass:[NSNull class]]) {
                [self.textures unregisterTexture:[textureId integerValue]];
                // 延迟一小段时间，否则可能会crash，因为flutter还可能调copyPixelBuffer
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t) (3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    NormalImageRenderWorker *worker = self.workers[textureId];
                    [[OpenGLRender sharedRenderer] disposeWorker:worker];
                    [self.workers removeObjectForKey:textureId];
                });

            }
            result(nil);

        } else {
            result(FlutterMethodNotImplemented);
        }
    });
}

@end
