import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef void FlutterImageViewCreatedCallback(ImageViewController controller);

class TipFlutterImageView extends StatefulWidget {
  const TipFlutterImageView(
      {Key key,
      this.url,
      this.width,
      this.height,
      this.aspectRatio,
      this.loading = const CupertinoActivityIndicator(),
      this.centerCrop = true})
      : super(key: key);
  final String url;
  final double width;
  final double height;
  final double aspectRatio;
  final Widget loading;
  final bool centerCrop; //是否剧中裁剪

  @override
  State<StatefulWidget> createState() => _TipFlutterImageViewState();
}

class _TipFlutterImageViewState extends State<TipFlutterImageView> {
  ImageViewController _controller;
  Stream<int> _stream;
  int textureId = -1;

  @override
  Widget build(BuildContext context) {
    return Texture(
      textureId: textureId,
    );
  }

  @override
  void initState() {
    super.initState();
    _controller = new ImageViewController.withUrl(widget.url, (_controller) {
      setState(() {
        textureId = _controller.textureId;
        _stream = _controller.stream;
      });
    });
    _controller.initialize(
        width: widget.width,
        height: widget.height,
        aspectRatio: widget.aspectRatio,
        centerCrop: widget.centerCrop);
  }

  @override
  void dispose() {
    //不加上这个会内存泄漏
    _controller.dispose();
    super.dispose();
  }
}

class ImageViewController {
  int textureId;
  MethodChannel _channel;
  Stream<int> stream;

  String url;
  bool disposed = false;
  FlutterImageViewCreatedCallback onImageViewCreated;

  ImageViewController.withUrl(
      String url, FlutterImageViewCreatedCallback onImageViewCreated) {
    this.url = url;
    this.onImageViewCreated = onImageViewCreated;
//    stream = new EventChannel('com.tencent.igame/flutter_image_view_event')
//        .receiveBroadcastStream(url)
//        .map((dynamic event) => _parseState(event));
//    stream.listen((data) {
//      print(
//          "ImageRenderWorker sendLoadSuccess()  listen called sink $stream  imageUrl  $url  data  $data");
//    });
    _channel = new MethodChannel('com.tencent.igame/flutter_image_view_method');
  }

  Future initialize(
      {double width,
      double height,
      double aspectRatio,
      bool centerCrop = true}) async {
    textureId = await _channel.invokeMethod('create', {
      'width': width,
      'height': height,
      'url': this.url,
      'aspectRatio': aspectRatio,
      'centerCrop': centerCrop
    });

    if (disposed) {
      this.dispose();
    } else {
      this.onImageViewCreated(this);
    }
  }

  void dispose() {
    _channel.invokeMethod('dispose', {'textureId': textureId});
    disposed = true;
  }

  bool get isInitialized => textureId != null;

  int _parseState(dynamic event) {
    return event;
  }
}
